
/*
 * Copyright (c) Developers of Binary Utility
 * Redistributable and modifiable. Please see license.readme for details.
 *
 * ammendment history
 *      <date>			        <changed by>		<summary of changes>
 * 26 November 2004  	    Jonee Ryan Ty		First documented history
 *
 */

#include <stdio.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
	// if MY_VERSION is not zero, it would overwrite an existing file
#define MY_VERSION 0
enum boolean {FALSE = 0, TRUE};

int error = FALSE;

int main(int argc, char* argv[]) {

	FILE *outFile;
	int exists(char *), i = 0;
	int isLeft = FALSE, offset = 0, base = 10, inputLen, outputLen, endBase = 10, endOffset = 0, inputNotExists = FALSE, outputExists = FALSE, nBytes = FALSE;
	char *file, *result, *myRead(char *, int *);
		/*do a lot of error checking */
	if ((argc != 5) && (argc != 6)) {	// parameter count
		fprintf(stderr, "four (or five) parameters expected\n");
		fprintf(stderr, "<file to write> <LEFT, RIGHT, or MIDDLE> <offset> <file to read> {end offset expected when MIDDLE used} \n");
		exit(1);
	}

	if (!exists(argv[4])) { // if input file does not exist
		fprintf(stderr, "file does not exist: %s\n", argv[4]);
		error = TRUE;
		inputNotExists = TRUE;
	}

	if (exists(argv[1])) { // if output file exists
		outputExists = TRUE;

		if (!MY_VERSION) {
			fprintf(stderr, "file exists: %s\n", argv[1]);
			error = TRUE;
		}
	}

		/* check whether the second paramete is either LEFT, right or midDle case insensitive */
	if (strcasecmp("MIDDLE", argv[2]) == 0) {

		if (argc == 5) {
			fprintf(stderr, "end offset expected\n");
			error = TRUE;
		}

	} else {

		if (argc == 6) {
			fprintf(stderr, "five parameters expected\n");
			printf("<file to write> <LEFT, RIGHT, or MIDDLE> <offset> <file to read> {end offset expected when MIDDLE used} \n");
			error = TRUE;
		}

		if (strcasecmp("LEFT", argv[2]) == 0) {
			isLeft = TRUE;

		} else if (strcasecmp("RIGHT", argv[2]) == 0) {
			isLeft = FALSE;

		} else {
			fprintf(stderr, "LEFT, RIGHT or MIDDLE?\n");
			printf("<file to write> <LEFT, RIGHT, or MIDDLE> <offset> <file to read> {end offset expected when MIDDLE used} \n");
			error = TRUE;
		}
	}

		/* check for base of offset */
	if (argv[3][0] == '0') {

		if (argv[3][1] == 'x' || argv[3][1] == 'X')
			base = 16;

		else if (argv[3][1] == '0')
			base = 8;
						
		else
			base = 10;
	} else {
		base = 10;
	}

	offset = (int)strtol(argv[3], NULL, base);

	if (argc == 6) {
		/* check for base of end offset */
		if (argv[5][0] == '0') {

			if (argv[5][1] == 'x' || argv[5][1] == 'X')
				endBase = 16;

			else if (argv[5][1] == '0')
				endBase = 8;

			else if (argv[5][1] == '+') {
				base = 10;
				nBytes = strlen(argv[5]);	// nBytes = true
			
				for (i = 2; i < nBytes; i++)
					argv[5][i - 2] = argv[5][i];
			
				argv[5][nBytes - 2] = '\0';
				
				nBytes = TRUE;	// just to reiterate this is boolean			

			} else
				endBase = 10;
		} else {
			endBase = 10;
		}

		endOffset = (int)strtol(argv[5], NULL, endBase);
	}

	if ((offset < 0) || (endOffset < 0)) {
		fprintf(stderr, "offset (and or end offset) negative?\n");
		error = TRUE;
	}

	if (strcmp(argv[1], argv[4]) == 0) {
		fprintf(stderr, "input file same as output\n");
		error = TRUE;
	}

	if (!inputNotExists) {
		file = myRead(argv[4], &inputLen);	// read file into the array

		if (file == NULL) {
			error = TRUE;

		} else {
			if (nBytes) {
				
				if (offset >= inputLen) {
					free(file);
					fprintf(stderr, "offset too big?\n");
					error = TRUE;
				}				
				
				if (inputLen < offset + endOffset) {
					
					free(file);
					fprintf(stderr, "offset + nBytes too big?\n");
					error = TRUE;				
				}

			} else {	
		
				if (offset >= inputLen) {
					free(file);
					fprintf(stderr, "offset too big?\n");
					error = TRUE;
				}
				
				if (endOffset >= inputLen) {
					free(file);
					fprintf(stderr, "end offset too big?\n");
					error = TRUE;
				}

				if ((offset > endOffset) && (argc == 6)) {
					free(file);
					fprintf(stderr, "offset bigger than end offset\n");
					error = TRUE;
				}
			}
		}
	}

	if (error)
		exit(1);
		/* no more error checking would happen from here on*/

	if ((offset != 0) && (base != 10))
		printf("base %d found for offset\n", base);

	printf("offset is %d\n", offset);

	if (argc == 5) {
		printf("reading to the %s\n", (isLeft)? "LEFT": "RIGHT");
	
	} else if (argc == 6) {
		if (nBytes) {
			printf("reading %i bytes\n", endOffset);
			
		} else {
			if ((endOffset != 0) && (endBase != 10))
				printf("base %d found for end offset\n", endBase);

			printf("end offset is %d\n", endOffset);
		}
	}

	outFile = fopen(argv[1], "w");
		/* compute for outputLen*/
	if (argc == 6) {
		if (nBytes)
			outputLen = endOffset;
		
		else
			outputLen = endOffset - offset + 1;

	} else {
		if (isLeft)
			outputLen = offset + 1;
		else
			outputLen = inputLen - offset;
	}

	result = (char *)malloc(outputLen);
		/* derive our desired output */

	if (isLeft) {
		for (i = 0; i < outputLen; i++)
			result[i] = file[i];

	} else {	// if RIGHT or MIDDLE		
		for (i = 0; i < outputLen; i++)
			result[i] = file[offset + i];
	}

	if (outputLen != 0)
		fwrite(result, 1, outputLen, outFile);
		
	fclose(outFile);

	free(result);
	free(file);
}

/**
 * check whether the file exists
 */
int exists(const char *outFileName) {
	FILE *tmp;
	tmp = fopen(outFileName, "r");

	if (tmp != NULL) {
		fclose(tmp);
		return TRUE;
	}

	return FALSE;
}

/**
 * read the file into a character array res, returns the character array (or NULL on error)
 * stores the size (or -1 on file descriptor error) on given parameter int pointer
 */
char *myRead(const char *fileName, int *size) {

	int fd;
	char *res;

	struct stat st;

	stat(fileName, &st);

	if((fd = open(fileName, O_RDONLY)) == -1) {
		fprintf(stderr, "file descriptor error for %s\n", fileName);
		*size = -1;
		return NULL;
	}

	res = (char *)malloc(st.st_size);

	read(fd, res, st.st_size);
	close(fd);

	*size = st.st_size;

	return res;
}







